from decimal import Decimal
import unittest

from pyson.ObjectMapper import ObjectMapper

from geniusweb.actions.Action import Action
from geniusweb.actions.Offer import Offer
from geniusweb.actions.PartyId import PartyId
from geniusweb.issuevalue.Bid import Bid
from geniusweb.issuevalue.DiscreteValue import DiscreteValue
from geniusweb.issuevalue.NumberValue import NumberValue


class BidTest (unittest.TestCase) :
    
    def testBadValue(self):
        self.assertRaises(ValueError, lambda:Bid({"iss":3}))

    def testSerialize(self):
        
        actor=PartyId("myid")
        # HACK currently bid requires all values same type
        bid=Bid({'fte':NumberValue(Decimal(3)), 'leasecar':DiscreteValue('yes')})
        offer=Offer(actor, bid)
        
        pyson=ObjectMapper()
        print(str(pyson.toJson(offer)))
        self.assertEqual({'Offer': {'actor': 'myid', 'bid': {'issuevalues':{'fte': 3, 'leasecar': 'yes'}}}},\
                        pyson.toJson(offer))
        
    def testDeserialize(self):
        actor=PartyId("myid")
        # HACK currently bid requires all values same type
        bid=Bid({'fte':NumberValue(Decimal(3)), 'leasecar':DiscreteValue('yes')})
        offer=Offer(actor, bid)
        pyson=ObjectMapper()

        jsonoffer= pyson.toJson(offer)
        self.assertEqual(offer, pyson.parse(jsonoffer, Action))
    
    def testNonValue(self):
        self.assertRaises(ValueError, lambda: Bid({"a":12}))